{% extends 'layout.html' %}
{% block title %}Login{% endblock %}
{% block body %}
<style>
    .new-character-creation {
        width:100%;
    }

    .new-character-creation li {
        margin-bottom: 20px;
    }


</style>
<div class="mdl-layout mdl-js-layout">
    <!-- Always shows a header, even in smaller screens. -->
    <div class="mdl-layout mdl-js-layout mdl-layout--fixed-header">
        <header class="mdl-layout__header">
            <div class="mdl-layout__header-row">
            <!-- Title -->
                <span class="mdl-layout-title">Nekøyume</span>
            </div>
        </header>
        <main class="mdl-layout__content">
            <div class="mdl-grid">
                <div class="mdl-cell mdl-cell--2-col">
                </div>
                <div class="mdl-cell mdl-cell--8-col">
                    <div class="mdl-card new-character-creation mdl-shadow--2dp">
                        <div class="mdl-card__title mdl-card--expand">
                            <h2 class="mdl-card__title-text">
                            👶
                            {% trans %}Currently creating a new character...{% endtrans %}
                            </h2>
                        </div>
                        <div class="mdl-card__supporting-text">
                            <p>
                            {% trans %}Hello! Welcome to the world of Nekoyume. We are currently creating a novice character. <br>This process will take about 15 seconds.{% endtrans %}</p>
                            <h5>{% trans %}Know in advance!{% endtrans %}</h5>
                            {% include 'basic_info.html' %}
                            <input type="hidden" name="new_move_id" value="{{ move.id }}">
                        </div>
                    </div>
                </div>
            </div>
        </main>
    </div>
</div>
<script>
var interval = setInterval(function() {
    $.getJSON('/moves/'+$('input[name=new_move_id]').val(), function(data) {
        if(data.move.block) {
            location.href='{{ url_for("game.get_dashboard") }}';
        }
    });
}, 3000);
</script>
{% endblock %}
