{% trans %}
<ul>
    <li>Nekoyume uses Dungeon World rules. <br> Familiarize yourself with the basic rules of Dungeon World so that you can play the game better fun.<br>
        <a href="https://www.dungeonworldsrd.com/" target="_blank">→ Learn Dungeon World</a></li>

    <li> Currently, only novice job is fixed without choosing a job in the preview network. <br> In future, the mainnet will be able to select all jobs offered by Dungeon World. </li>
    <li> It takes about 15 seconds for the player activation. <br> But sometimes it takes longer, so please be patient. 🙏 </li>
    <li> If you have Gold, you can become more powerful player. <br>
        <a href="https://nekoyu.me/token.html" target="_blank">→ Buy Gold</a></li>
</ul>
{% endtrans %}
